(function() {
  var colors, cp, createNewProject, dirWatcher, echoUsageAndExit, fs, helper, path, readTemplateFileSync, setAppData, util, validateDirectoryDoesntExist, validateNumberOfParametersForNewProjects;
  colors = require('colors');
  cp = require('child_process');
  dirWatcher = require('dir-watcher');
  helper = require('./helper');
  fs = require('fs');
  path = require('path');
  util = require('util');
  createNewProject = function(appData) {
    util.log('Creating project...');
    fs.mkdirSync("" + appData.name + "/", 0744);
    fs.writeFile("" + appData.name + "/.gitignore", readTemplateFileSync('gitignore'));
    fs.writeFile("" + appData.name + "/.npmignore", readTemplateFileSync('npmignore'));
    if (appData.task === 'cli') {
      fs.mkdirSync("" + appData.name + "/bin/", 0744, function() {});
      fs.writeFile("" + appData.name + "/bin/" + appData.name, "#!/usr/bin/env node\nrequire('../lib/" + appData.name + "').start();", function(err) {
        return fs.chmod("" + appData.name + "/bin/" + appData.name, 0755);
      });
    }
    fs.writeFile("" + appData.name + "/Cakefile", "cp = require 'child_process'\nutil = require 'util'\n\n# globals\ntaskRunning = {}\ntaskDone = {}\n\n# private functions\nrun = (cmd, args, finishedCallback) ->\n	p = cp.spawn cmd, args\n	p.stdout.on 'data', (data) ->\n		process.stdout.write data\n	p.stderr.on 'data', (data) ->\n		process.stderr.write data\n	p.on 'exit', (exitCode) ->\n		util.log cmd + ' process exited with code ' + exitCode\n		finishedCallback() if finishedCallback?\n\ntaskRunOnce = (name, description, callback) ->\n	task name, description, (options) ->\n		return if taskRunning[name]?\n		taskRunning[name] = true\n		callback options, ->\n			taskDone[name] = true\n\nvalidateRootAccess = ->\n	isRoot = (process.env['USER'] == 'root')\n	if !isRoot\n		console.log 'Must be run as root'\n		process.exit -1\n\nwaitUntilTaskDone = (task, callback) ->\n	intervalId = setInterval ->\n		if taskDone[task]\n			clearInterval intervalId\n			callback()\n	, 200\n\n# tasks\ntaskRunOnce 'build', 'compile all files in src/ to be placed in lib/', (options, setDoneFlag) ->\n	invoke 'clean'\n	waitUntilTaskDone 'clean', ->\n		util.log 'Building...'\n		run 'coffee',['-c', '-o', 'lib/', 'src/'], setDoneFlag\n\ntaskRunOnce 'clean', 'delete everything in lib/', (options, setDoneFlag) ->\n	util.log 'Cleaning...'\n	cp.exec 'rm -rf lib/*', setDoneFlag\n" + appData.cakeInstallAndUninstallText);
    fs.mkdirSync("" + appData.name + "/lib/", 0744);
    fs.mkdirSync("" + appData.name + "/src/", 0744, function() {});
    fs.writeFile("" + appData.name + "/src/" + appData.name + ".coffee", appData.mainCode);
    fs.mkdirSync("" + appData.name + "/test/", 0744, function() {});
    fs.writeFile("" + appData.name + "/test/" + appData.name + ".test.coffee", "app = require '../src/" + appData.name + "'\nshould = require 'should'\n\nexports.testTodo = ->\n	1.should.be.equal 1");
    fs.writeFile("" + appData.name + "/README.md", "# " + appData.name + "\n" + appData.description + "\n\nEnjoy!\n\n" + appData.author);
    return fs.writeFile("" + appData.name + "/package.json", "{\n  \"name\": \"" + appData.name + "\",\n  \"description\": \"" + appData.description + "\",\n  \"author\": \"" + appData.author + "\",\n  \"version\": \"0.0.1\",\n  \"main\": \"lib/" + appData.name + "\",\n  \"licenses\": [{\n    \"type\": \"MIT\",\n    \"url\": \"http://opensource.org/licenses/mit-license.php\"\n  }],\n  \"engines\": {\n    \"node\": \">=0.4.5\"\n  }" + appData.additionalPackageCode + "\n}", function() {
      process.chdir(appData.name);
      return helper.runCommand('npm', ['install'], function() {
        return helper.runCommand('npm', ['install', 'should'], function() {
          return util.log('Project created.'.green.bold);
        });
      });
    });
  };
  echoUsageAndExit = function() {
    console.log('Usage:');
    console.log('coffee-new autotest                          : watch src/ & test/ directories & run tests based on files that have changed');
    console.log('coffee-new cli <name> <author> <description> : create CLI project');
    console.log('coffee-new lib <name> <author> <description> : create library project');
    console.log('coffee-new publish                           : publish to NPM');
    console.log('coffee-new test                              : run entire test suite');
    return process.exit(-1);
  };
  readTemplateFileSync = function(filename) {
    return fs.readFileSync(path.resolve(__dirname, "../templates/" + filename));
  };
  setAppData = function(task) {
    return {
      task: task,
      name: process.argv[3],
      author: process.argv[4],
      description: process.argv[5]
    };
  };
  validateDirectoryDoesntExist = function(name) {
    if (path.existsSync("" + name + "/")) {
      console.warn("Error: Directory " + name + " already exists");
      return process.exit(-1);
    }
  };
  validateNumberOfParametersForNewProjects = function() {
    if (process.argv.length !== 6) {
      console.warn('Error: Incorrect number of parameters');
      return echoUsageAndExit();
    }
  };
  exports.start = function() {
    var appData, task;
    task = process.argv[2];
    switch (task) {
      case 'autotest':
        helper.validateCoffeeDirectorySync();
        util.log('Watching...');
        return helper.autoTest();
      case 'cli':
        validateNumberOfParametersForNewProjects();
        appData = setAppData(task);
        appData.mainCode = readTemplateFileSync('cli-appcode.coffee');
        validateDirectoryDoesntExist(appData.name);
        appData.cakeInstallAndUninstallText = "\ntaskRunOnce 'install', 'build & run npm -g install', (options, setDoneFlag) ->\n	validateRootAccess()\n	invoke 'build'\n	waitUntilTaskDone 'build', ->\n		util.log 'Installing...'\n		run 'npm', ['-g', 'install'], setDoneFlag\n\ntaskRunOnce 'uninstall', 'run npm -g uninstall', (options, setDoneFlag) ->\n	validateRootAccess()\n	util.log 'Uninstalling...'\n	run 'npm', ['-g', 'uninstall', '" + appData.name + "'], setDoneFlag";
        appData.additionalPackageCode = ",\n  \"dependencies\": {\n    \"colors\": \"=0.5.0\",\n    \"commander\": \"=0.1.0\"\n  },\n  \"bin\": {\n    \"" + appData.name + "\": \"./bin/" + appData.name + "\"\n  }";
        return createNewProject(appData);
      case 'lib':
        validateNumberOfParametersForNewProjects();
        appData = setAppData(task);
        appData.mainCode = fs.readFileSync(path.resolve(__dirname, '../templates/lib-appcode.coffee'));
        validateDirectoryDoesntExist(appData.name);
        appData.cakeInstallAndUninstallText = "\ntaskRunOnce 'install', 'run npm install', (options, setDoneFlag) ->\n	run 'npm', ['install'], setDoneFlag";
        appData.additionalPackageCode = "";
        return createNewProject(appData);
      case 'publish':
        helper.validateCoffeeDirectorySync();
        return helper.publish();
      case 'test':
        helper.validateCoffeeDirectorySync();
        return helper.runTestSuite();
      default:
        console.warn('Error: First parameter <task> needs to be autotest, cli, lib, publish or test.');
        return echoUsageAndExit();
    }
  };
}).call(this);
