(function() {
  var changeFileExtension, cleanAndBuild, colors, cp, dirWatcher, fs, getTestSuiteFiles, isDirectorySync, isFileSync, path, publish, runCommand, runTestFile, runTestSuite, util, validateCoffeeDirectorySync;
  colors = require('colors');
  cp = require('child_process');
  dirWatcher = require('dir-watcher');
  fs = require('fs');
  path = require('path');
  util = require('util');
  cleanAndBuild = function(finishedCallback) {
    util.log('Cleaning...');
    return cp.exec('rm -rf lib/*', function() {
      util.log('Building...');
      return runCommand('coffee', ['-c', '-o', 'lib/', 'src/'], function(exitCode) {
        if (finishedCallback != null) {
          return finishedCallback(exitCode);
        }
      });
    });
  };
  changeFileExtension = function(filePath, oldExtension, newExtension) {
    if (path.extname(filePath) !== oldExtension) {
      throw 'changeFileExtension Error: Path ' + filePath + ' does not have the extension ' + oldExtension + '.';
    }
    return path.resolve(path.dirname(filePath), path.basename(filePath, oldExtension) + newExtension);
  };
  getTestSuiteFiles = function() {
    var cwd, entry, _i, _len, _ref, _results;
    cwd = process.cwd();
    _ref = dirWatcher.walkDirectoryTreeSync('test');
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      entry = _ref[_i];
      if (dirWatcher.isFileSync(entry)) {
        _results.push(dirWatcher.getRelativePath(cwd, entry));
      }
    }
    return _results;
  };
  isDirectorySync = function(dir) {
    if (path.existsSync(dir)) {
      return fs.statSync(dir).isDirectory();
    } else {
      return false;
    }
  };
  isFileSync = function(dir) {
    if (path.existsSync(dir)) {
      return fs.statSync(dir).isFile();
    } else {
      return false;
    }
  };
  publish = function() {
    return runTestSuite(function(exitCode) {
      if (exitCode === 0) {
        return cleanAndBuild(function() {
          util.log('Publishing...');
          return runCommand('npm', ['publish', '--force']);
        });
      }
    });
  };
  runCommand = function(cmd, args, finishedCallback) {
    var p;
    p = cp.spawn(cmd, args);
    p.stdout.on('data', function(data) {
      return process.stdout.write(data);
    });
    p.stderr.on('data', function(data) {
      return process.stderr.write(data);
    });
    return p.on('exit', function(exitCode) {
      var exitString;
      exitString = cmd + ' process exited with code ' + exitCode;
      if (exitCode !== 0) {
        util.log(exitString.red);
      } else {
        util.log(exitString.green);
      }
      if (finishedCallback != null) {
        return finishedCallback(exitCode);
      }
    });
  };
  runTestFile = function(file) {
    return runCommand('expresso', [file]);
  };
  runTestSuite = function(finishedCallback) {
    util.log('Testing...');
    return runCommand('expresso', getTestSuiteFiles(), finishedCallback);
  };
  validateCoffeeDirectorySync = function(dir) {
    var isCoffeeDir;
    isCoffeeDir = isDirectorySync(path.resolve(dir, 'lib')) && isDirectorySync(path.resolve(dir, 'src')) && isDirectorySync(path.resolve(dir, 'test')) && isFileSync(path.resolve(dir, 'Cakefile')) && isFileSync(path.resolve(dir, 'package.json'));
    if (!isCoffeeDir) {
      console.log('Must be in a CoffeeScript project directory');
      process.exit(-1);
    }
    return isCoffeeDir;
  };
  exports.autoTest = function(persist, runTestFileCallback) {
    var cwd, srcWatcher, testWatcher;
    if (persist == null) {
      persist = true;
    }
    if (!(runTestFileCallback != null)) {
      runTestFileCallback = function(relativeTestPath, exists) {
        if (exists) {
          util.log('Running test file '.yellow + relativeTestPath.yellow.bold);
          return runTestFile(relativeTestPath);
        } else {
          return util.log(('Test file ' + relativeTestPath + " doesn't exist.").red);
        }
      };
    }
    cwd = process.cwd();
    dirWatcher.setup(persist);
    srcWatcher = dirWatcher.create(function(entry) {
      var testFile;
      if (path.extname(entry) === '.coffee') {
        testFile = dirWatcher.getMirrorPath(cwd, entry, 'src', 'test');
        testFile = changeFileExtension(testFile, '.coffee', '.test.coffee');
        return path.exists(testFile, function(exists) {
          var relativeTestPath;
          relativeTestPath = dirWatcher.getRelativePath(cwd, testFile);
          return runTestFileCallback(relativeTestPath, exists);
        });
      }
    });
    srcWatcher.watchDirectoryTreeSync('src');
    testWatcher = dirWatcher.create(function(entry) {
      var relativeTestPath;
      if (path.extname(entry) === '.coffee') {
        relativeTestPath = dirWatcher.getRelativePath(cwd, entry);
        return runTestFileCallback(relativeTestPath, true);
      }
    });
    return testWatcher.watchDirectoryTreeSync('test');
  };
  exports.changeFileExtension = changeFileExtension;
  exports.getTestSuiteFiles = getTestSuiteFiles;
  exports.publish = publish;
  exports.runCommand = runCommand;
  exports.runTestFile = runTestFile;
  exports.runTestSuite = runTestSuite;
  exports.validateCoffeeDirectorySync = validateCoffeeDirectorySync;
}).call(this);
